#include <SPI.h>
#include "Config.h"

#if defined(flg_serial_debug) && defined(__AVR_ATtiny85__)
#include <SendOnlySoftwareSerial.h>
#endif

#define NUM_PIXS (ARRAY_WIDTH * ARRAY_HEIGHT)

void setup()
{
	//set pin I/O direction
	pin_nCS_Mode_OUTPUT;
	pin_nCS_HIGH;
	pin_led_Mode_OUTPUT;
	pin_led_HIGH;
	pin_TRIG_Mode_INPUT;
	//pinMode(4, INPUT);

	//initialize SPI
#if !defined(flg_software_spi) && !defined(__AVR_ATtiny85__)
	SPI.begin();
	SPI.setDataMode(SPI_MODE3);
	//SPI.setClockDivider(SPI_CLOCK_DIV32);
#else
	pin_nClock_Mode_OUTPUT;
	pin_nClock_HIGH;
	pin_SDIO_Mode_OUTPUT;
	pin_SDIO_LOW;
#endif

#if defined(flg_serial_debug)
#if !defined(__AVR_ATtiny85__)
	Serial.begin(230400);
#else
	MyDbgSerial.begin(115200);
	//MyDbgSerial.println(F("\nDebug enabled"));
#endif
#endif
	Serial.begin(230400);
	delay(1000);
	ADNS_reset();
}


void loop()
{
#ifndef flg_serial_debug
	//-----------------------------------------------------------------------
	byte maxPix, minPix, srfqual;
	uint16_t counter = 0;
	int16_t tempExtrem = 20000;
	int16_t extrem = 20000;

	union {
		uint16_t int16;
		struct {
			uint8_t shutLow;
			uint8_t shutUpp;
		} bytes;
	} shutt;

	typedef enum { ACTIVE, AWAITING, TRIGGERED } mode_type;
	mode_type mode = AWAITING;

	while (1)
	{
		//srfqual = ADNS_read(squal);
		maxPix = ADNS_read(Maximum_Pixel);
		//minPix = ADNS_read(Minimum_Pixel);
		//shutt.bytes.shutLow = ADNS_read(Shutter_Lower);
		//shutt.bytes.shutUpp = ADNS_read(Shutter_Upper);
		//data = ADNS_read(Pixel_Sum);

		if (mode == AWAITING) {
			if (maxPix == 63) {
				mode = ACTIVE;
			}
			else {
				pin_led_LOW;
			}
		}
		else if (mode == ACTIVE) {
			if (maxPix < 58) {
				mode = TRIGGERED;
				pin_led_HIGH;
			}
		}
		else {
			if (counter++ > 20) {
				mode = AWAITING;
				pin_led_LOW;
				counter = 0;
			}
		}

	}
	//-----------------------------------------------------------------------
#endif
#if flg_serial_debug ==1
	//-----------------------------------------------------------------------
	byte Frame[(ARRAY_WIDTH * ARRAY_HEIGHT)+1+6];

	Frame[(ARRAY_WIDTH * ARRAY_HEIGHT)+1+6] = 0xff;	// frame data is 0..63 or 0..127 so delimiter 255 is safe
	while(1)
	{
		/*#ifdef flg_ADNS_type_ADNS_2610
				ADNS_write(0x00,0x01); //Always awake
				#endif
				#ifdef flg_ADNS_type_ADNS_5020
				//		ADNS_write(0x0d, 0x01);	// Set 1000cpi resolution
				#endif*/
		// grab frame and write to serial
		//reset the pixel grab counter
		ADNS_write(Pixel_Grab,0x00);

		int count = 0;

		for (count=0;count < (ARRAY_WIDTH * ARRAY_HEIGHT); count++)
		{
			while(1) {Frame[count] = ADNS_read(Pixel_Grab); if(Frame[count] & Pixel_data_valid) break;}
			Frame[count] &= Mask_pixel_value;	// only n bits are used for data
		}

		Frame[NUM_PIXS] = ADNS_read(squal);
		Frame[NUM_PIXS + 1] = ADNS_read(Maximum_Pixel);
		Frame[NUM_PIXS + 2] = ADNS_read(Minimum_Pixel);
		Frame[NUM_PIXS + 3] = ADNS_read(Pixel_Sum);
		Frame[NUM_PIXS + 4] = ADNS_read(Shutter_Upper);
		Frame[NUM_PIXS + 5] = ADNS_read(Shutter_Lower);

#if !defined(__AVR_ATtiny85__)
		Serial.write(Frame, (ARRAY_WIDTH * ARRAY_HEIGHT)+1+6); // send frame in raw format
#else
		MyDbgSerial.write(Frame, (ARRAY_WIDTH * ARRAY_HEIGHT)+1);
#endif
	}
	//-----------------------------------------------------------------------
#endif
#if flg_serial_debug ==2
	//-----------------------------------------------------------------------
	byte Frame[(ARRAY_WIDTH * ARRAY_HEIGHT) + 1 + 6];

	Frame[(ARRAY_WIDTH * ARRAY_HEIGHT) + 1 + 6] = 0xff;	// frame data is 0..63 or 0..127 so delimiter 255 is safe

	byte maxPix, minPix, sumPix, srfsqual;
	union {
		uint16_t int16;
		struct {
			uint8_t shutLow;
			uint8_t shutUpp;
		} bytes;
	} shutt;

	while (1)
	{
		ADNS_write(Pixel_Grab, 0x00);
		//srfsqual = ADNS_read(squal);
		maxPix = ADNS_read(Maximum_Pixel);
		//minPix = ADNS_read(Minimum_Pixel);
		//shutt.bytes.shutLow = ADNS_read(Shutter_Lower);
		//shutt.bytes.shutUpp = ADNS_read(Shutter_Upper);
		//sumPix = ADNS_read(Pixel_Sum);

		//if (srfsqual > 80 && (maxPix - minPix) > 59) {
		if (maxPix - minPix > 50) {
			pin_led_HIGH;


			int count = 0;

			for (count = 0; count < (ARRAY_WIDTH * ARRAY_HEIGHT); count++)
			{
				while (1) { Frame[count] = ADNS_read(Pixel_Grab); if (Frame[count] & Pixel_data_valid) break; }
				Frame[count] &= Mask_pixel_value;	// only n bits are used for data
			}
			Frame[NUM_PIXS] = srfsqual;
			Frame[NUM_PIXS + 1] = maxPix;
			Frame[NUM_PIXS + 2] = minPix;
			Frame[NUM_PIXS + 3] = ADNS_read(Pixel_Sum);
			Frame[NUM_PIXS + 4] = ADNS_read(Shutter_Upper);
			Frame[NUM_PIXS + 5] = ADNS_read(Shutter_Lower);

#if !defined(__AVR_ATtiny85__)
			Serial.write(Frame, (ARRAY_WIDTH * ARRAY_HEIGHT) + 1 + 6); // send frame in raw format
#else
			MyDbgSerial.write(Frame, (ARRAY_WIDTH * ARRAY_HEIGHT) + 1);
#endif
		}
		else {
			pin_led_LOW;
		}
	}
	//-----------------------------------------------------------------------
#endif
#if flg_serial_debug ==3
	//-----------------------------------------------------------------------
	byte Frame[(ARRAY_WIDTH * ARRAY_HEIGHT)+1];

	Frame[(ARRAY_WIDTH * ARRAY_HEIGHT)+1] = 0xff;	// frame data is 0..63 or 0..127 so delimiter 255 is safe
	while(1)
	{
		Frame[0] = ADNS_read(squal);
		Frame[1] = ADNS_read(Maximum_Pixel);
		Frame[2] = ADNS_read(Minimum_Pixel);
		Frame[3] = ADNS_read(Pixel_Sum);
		Frame[4] = ADNS_read(Shutter_Upper);
		Frame[5] = ADNS_read(Shutter_Lower);

#if !defined(__AVR_ATtiny85__)
		Frame[1] >120 ? pin_led_HIGH : pin_led_LOW;
		Serial.println 			("squal: "	+ String(Frame[0]) + \
			"\tMax_Pix:"	+ String(Frame[1]) + \
			"\tMin_Pix:"	+ String(Frame[2]) + \
			"\tPix_Sum:"	+ String(Frame[3]) + \
			"\tShutter:"	+ String((uint16_t)Frame[4]*256 + Frame[5]));
		delay(50);
#else
		MyDbgSerial.print	(F("squal: "));		MyDbgSerial.write(ByteToAscii_h(Frame[0])); MyDbgSerial.write(ByteToAscii_l(Frame[0]));
		MyDbgSerial.print	(F("\tMax_Pix:"));	MyDbgSerial.write(ByteToAscii_h(Frame[1])); MyDbgSerial.write(ByteToAscii_l(Frame[1]));
		MyDbgSerial.print	(F("\tMin_Pix:"));	MyDbgSerial.write(ByteToAscii_h(Frame[2])); MyDbgSerial.write(ByteToAscii_l(Frame[2]));
		MyDbgSerial.print	(F("\tPix_Sum:"));	MyDbgSerial.write(ByteToAscii_h(Frame[3])); MyDbgSerial.write(ByteToAscii_l(Frame[3]));
		MyDbgSerial.print	(F("\tShutter:"));	MyDbgSerial.write(ByteToAscii_h(Frame[4])); MyDbgSerial.write(ByteToAscii_l(Frame[4]));
		MyDbgSerial.write(ByteToAscii_h(Frame[5])); MyDbgSerial.write(ByteToAscii_l(Frame[5]));
		if (Frame[1] > 120)
		{
			MyDbgSerial.print(F("\tOk"));
		}
		MyDbgSerial.println(F(""));
		delay(100);	//задержка больше из-за низкой скорости Digispark
#endif
	}
	//-----------------------------------------------------------------------
#endif
#if flg_serial_debug ==4
	//-----------------------------------------------------------------------
	while(1)
	{
		unsigned int motion = 0;
		//check for movement
		motion = ADNS_read(Motion);
		if(motion > 127)
		{
			//there has been a motion! Print the x and y movements
			MyDbgSerial.println("X:" + String(ADNS_read(Delta_X)) + " Y:" + String(ADNS_read(Delta_Y)));
		}
	}
	//-----------------------------------------------------------------------
#endif
#if flg_serial_debug ==5
	//-----------------------------------------------------------------------
	byte Frame[(ARRAY_WIDTH * ARRAY_HEIGHT)+1];

	Frame[(ARRAY_WIDTH * ARRAY_HEIGHT)+1] = 0xff;	// frame data is 0..63 or 0..127 so delimiter 255 is safe
	int counter = 0;
	delay(1000);
	Serial.println("meas\tsqual\tMax_Pix\tMin_Pix\tPix_Sum\tShutter");
	while(1)
	{
		if (pin_TRIG_IN) {
			//if (digitalRead(4)) {
			//for (int i = 0; i < NUM_SAMPLES_PER_MEASURE; i++) {
			Frame[0] = ADNS_read(squal);
			Frame[1] = ADNS_read(Maximum_Pixel);
			Frame[2] = ADNS_read(Minimum_Pixel);
			Frame[3] = ADNS_read(Pixel_Sum);
			Frame[4] = ADNS_read(Shutter_Upper);
			Frame[5] = ADNS_read(Shutter_Lower);

			Serial.println(String(counter) + \
				"\t" + String(Frame[0]) + \
				"\t" + String(Frame[1]) + \
				"\t" + String(Frame[2]) + \
				"\t" + String(Frame[3]) + \
				"\t" + String((uint16_t)Frame[4] * 256 + Frame[5]));
			//}
			counter++;
			//delay(300);
		}

	}
	//-----------------------------------------------------------------------
#endif
}


void ADNS_reset(){

#ifdef flg_ADNS_type_ADNS_5020
	ADNS_write(0x3a, 0x5a);
	delay(1000);
	ADNS_write(0x0d, 0x01);	// Set 1000cpi resolution
	delay(1000);
#endif
#ifdef flg_ADNS_type_ADNS_2610
	//ADNS_write(0x00,0x80);
	delay(1000);
	ADNS_write(0x00, 0x01);
#endif
#ifdef flg_ADNS_type_ADNS_2620
	ADNS_write(0x00,0x80);
	delay(1000);
	ADNS_write(0x40, 0x01);
#endif
}

//-------------------------------------------------------
#if !defined(flg_software_spi) && !defined(__AVR_ATtiny85__)
void ADNS_write(byte address, byte data)
{
	// take the CS pin low to select the chip:
	pin_nCS_LOW;
	// send in the address and value via SPI:
	SPI.transfer(address | 0x80);
	SPI.transfer(data);
	// take the SS pin high to de-select the chip:
	pin_nCS_HIGH;
}

byte ADNS_read(byte address)
{
	byte data;
	// take the CS pin low to select the chip:
	pin_nCS_LOW;
	// send in the address and value via SPI:
	SPI.transfer(address);

	pin_SDIO_Mode_INPUT;
	delayMicroseconds(delay_tSRAD);
	data = SPI.transfer(0x00);
	pin_SDIO_Mode_OUTPUT;
	// take the SS pin high to de-select the chip:
	pin_nCS_HIGH;
	return data;
	}
//-------------------------------------------------------
#else
void ADNS_write(byte address, byte data)
{
	// take the CS pin low to select the chip:
	pin_nCS_LOW;

	// send in the address and value via SPI:
	address |= 0x80;	//признак записи адреса
	for (byte i = 0x80; i; i >>= 1)
	{
		pin_nClock_LOW;
		address & i ? pin_SDIO_HIGH : pin_SDIO_LOW;
		asm volatile ("nop");
		pin_nClock_HIGH;
	}

	//delayMicroseconds(1);

	for (byte i = 0x80; i; i >>= 1)
	{
		pin_nClock_LOW;
		data & i ? pin_SDIO_HIGH : pin_SDIO_LOW;
		asm volatile ("nop");
		pin_nClock_HIGH;
	}
	//t SWW. SPI Time between Write Commands
	delayMicroseconds(delay_tSWW);

	// take the SS pin high to de-select the chip:
	pin_nCS_HIGH;
}

byte ADNS_read(byte address)
{
	// take the CS pin low to select the chip:
	pin_nCS_LOW;

	address &= ~0x80;	//признак записи данных
	for (byte i = 0x80; i; i >>= 1)
	{
		pin_nClock_LOW;
		address & i ? pin_SDIO_HIGH : pin_SDIO_LOW;
		asm volatile ("nop");
		pin_nClock_HIGH;
	}

	// prepare io pin for reading
	pin_SDIO_Mode_INPUT;

	// t SRAD. SPI Read Address-Data Delay
	delayMicroseconds(delay_tSRAD);

	// read the data
	byte data = 0;
	for (byte i = 8; i; i--)
	{
		// tick, tock, read
		pin_nClock_LOW;
		asm volatile ("nop");
		pin_nClock_HIGH;
		data <<= 1;
		if (pin_SDIO_IN) data |= 0x01;
	}

	pin_nCS_HIGH;
	pin_SDIO_Mode_OUTPUT;

	// t SRW & t SRR = 1μs.
	delayMicroseconds(2);
	return data;
}
#endif
//-------------------------------------------------------

uint8_t ByteToAscii_h(uint8_t x)
{
	uint8_t tmpByte;

	tmpByte = (x >> 4) + 0x30;
	if (tmpByte > 0x39) { tmpByte += 0x07; }
	return tmpByte;
}
uint8_t ByteToAscii_l(uint8_t x)
{
	uint8_t tmpByte;

	tmpByte = (x & 0x0f) + 0x30;
	if (tmpByte > 0x39) { tmpByte += 0x07; }
	return tmpByte;
}


/*
Sketch uses 1 268 bytes (4%) of program storage space. Maximum is 30 720 bytes.
Global variables use 236 bytes (11%) of dynamic memory, leaving 1 812 bytes for local variables. Maximum is 2 048 bytes.

программный spi
Sketch uses 912 bytes (2%) of program storage space. Maximum is 30 720 bytes.
Global variables use 235 bytes (11%) of dynamic memory, leaving 1 813 bytes for local variables. Maximum is 2 048 bytes.

SoftSerial
Sketch uses 3 012 bytes (50%) of program storage space. Maximum is 6 012 bytes.
Global variables use 129 bytes of dynamic memory.
замена SoftSerial на SendOnlySoftwareSerial
Sketch uses 1 878 bytes (31%) of program storage space. Maximum is 6 012 bytes.
Global variables use 42 bytes of dynamic memory.
*/




